﻿/*
 Author: Wojciech Domski
 Email:  Wojciech.Domski@gmail.com
 WWW:    Domski.pl

 copyright 2013 Wojciech Domski

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SOS_HEADER
#define SOS_HEADER

/**
 * \brief Max event numebr
 *
 * Maximum number of events
 * which can be stored in
 * \link sos_event_array
 * sos_event_array \endlink .
 */
#define SOS_MAX_EVENT_NUMBER 10

/**
 * \brief Turned off event flag
 *
 * When event has set this flag it is inactive.
 * When event has flag value >= 0 event is active.
 */
#define SOS_EVENT_OFF (-1)

/**
 * \brief Do not force slot number
 *
 * Used in
 * \link sos_add_event sos_add_event \endlink .
 */
#define SOS_NEXT_FREE_SLOT (-1)

/**
 *	\brief SOS function call back typedef
 */
typedef void (*sos_function_callback_t)(void);

/**
 * \brief Initialize SysTick
 *
 *	This function is platform dependent
 *	It requires to initialize any counter which is
 *	capable of sending interrupt.
 */
void sos_init(void);

/**
 * \brief SOS callback
 *
 * This function should be called inside
 * counter (preferably SysTick) handler.
 * It runs the OS.
 */
void sos_handler(void);

/**
 * \brief Add event
 *
 * Adds event callback to SOS queue with required period.
 *
 * \param[in]  ptr 		Pointer to the event callback function.
 * \param[in]  period   Period in ticks. When sos_handler is
 * 						called every 1 ms then setting period to 2
 * 						the event callback function will
 * 						be called every 2 ms.
 *             slot		Preferred slot number. Can be SOS_NEXT_FREE_SLOT.
 *
 * \return     -1   -> no free slots
 *            >=0   -> assigned slot number
 */
int sos_add_event(sos_function_callback_t ptr, int period, int slot);

/**
 * \brief Set period for event
 *
 *
 * \param[in] period	New period in ticks
 * \param[in] slot		Slot number
 *
 */
void sos_set_period(int period, int slot);

/**
 * \brief Macro for disabling event
 *
 * \param[in] slot		Slot number
 */
#define sos_disable_event( slot) sos_set_period( SOS_EVENT_OFF, slot)

#endif

